@@ -49,7 +49,7 @@ def generateAssignmentAttributeCode(statement, emit, context):
 
     old_source_ref = context.setCurrentSourceCodeReference(
         value.getSourceReference()
-        if Options.isFullCompat()
+        if Options.is_fullcompat
         else statement.getSourceReference()
     )
 
@@ -85,7 +85,7 @@ def generateDelAttributeCode(statement, emit, context):
 
     old_source_ref = context.setCurrentSourceCodeReference(
         statement.subnode_expression.getSourceReference()
-        if Options.isFullCompat()
+        if Options.is_fullcompat
         else statement.getSourceReference()
     )
 
@@ -255,10 +255,10 @@ def generateAttributeCheckCode(to_name, expression, emit, context):
         expression=expression, emit=emit, context=context
     )
 
-    res_name = context.getIntResName()
+    res_name = context.getBoolResName()
 
     emit(
-        "%s = PyObject_HasAttr(%s, %s);"
+        "%s = HAS_ATTR_BOOL(%s, %s);"
         % (
             res_name,
             source_name,
@@ -269,7 +269,7 @@ def generateAttributeCheckCode(to_name, expression, emit, context):
     getReleaseCode(release_name=source_name, emit=emit, context=context)
 
     to_name.getCType().emitAssignmentCodeFromBoolCondition(
-        to_name=to_name, condition="%s != 0" % res_name, emit=emit
+        to_name=to_name, condition=res_name, emit=emit
     )
 
 
