@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,6 +25,7 @@ from nuitka.PythonVersions import python_version
 from .CodeHelpers import (
     generateChildExpressionsCode,
     generateExpressionCode,
+    withCleanupFinally,
     withObjectCodeTemporaryAssignment,
 )
 from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode
@@ -45,7 +46,7 @@ def generateBuiltinDictCode(to_name, expression, emit, context):
         seq_name = None
 
     with withObjectCodeTemporaryAssignment(
-        to_name, "dict_result", expression, emit, context
+        to_name, "dict_value", expression, emit, context
     ) as value_name:
 
         if expression.getNamedArgumentPairs():
@@ -94,30 +95,58 @@ def generateDictionaryCreationCode(to_name, expression, emit, context):
         to_name, "dict_result", expression, emit, context
     ) as value_name:
         _getDictionaryCreationCode(
-            to_name=value_name, pairs=expression.getPairs(), emit=emit, context=context
+            to_name=value_name,
+            pairs=expression.subnode_pairs,
+            emit=emit,
+            context=context,
         )
 
 
 def _getDictionaryCreationCode(to_name, pairs, emit, context):
+    # Detailed, and verbose code, pylint: disable=too-many-locals
+
+    pairs_count = len(pairs)
+
+    # Empty dictionaries should not get to this function, but be constant value instead.
+    assert pairs_count > 0
+
+    # Unique per dictionary, they might be nested, but used for all of them.
+    dict_key_name = context.allocateTempName("dict_key")
+    dict_value_name = context.allocateTempName("dict_value")
+
+    is_hashable_key = [pair.subnode_key.isKnownToBeHashable() for pair in pairs]
+
+    # Does this dictionary build need an exception handling at all.
+    if all(is_hashable_key):
+        for pair in pairs[1:]:
+            if pair.subnode_key.mayRaiseException(BaseException):
+                needs_exception_exit = True
+                break
+            if pair.subnode_value.mayRaiseException(BaseException):
+                needs_exception_exit = True
+                break
+        else:
+            needs_exception_exit = False
+    else:
+        needs_exception_exit = True
+
     def generateValueCode(dict_value_name, pair):
         generateExpressionCode(
             to_name=dict_value_name,
-            expression=pair.getValue(),
+            expression=pair.subnode_value,
             emit=emit,
             context=context,
         )
 
     def generateKeyCode(dict_key_name, pair):
         generateExpressionCode(
-            to_name=dict_key_name, expression=pair.getKey(), emit=emit, context=context
+            to_name=dict_key_name,
+            expression=pair.subnode_key,
+            emit=emit,
+            context=context,
         )
 
-    assert pairs
-
-    for count, pair in enumerate(pairs):
-        dict_key_name = context.allocateTempName("dict_key")
-        dict_value_name = context.allocateTempName("dict_value")
-
+    def generatePairCode(pair):
         # Strange as it is, CPython 3.5 and before evaluated the key/value pairs
         # strictly in order, but for each pair, the value first.
         if python_version < 350:
@@ -127,33 +156,50 @@ def _getDictionaryCreationCode(to_name, pairs, emit, context):
             generateKeyCode(dict_key_name, pair)
             generateValueCode(dict_value_name, pair)
 
-        if count == 0:
-            emit("%s = _PyDict_NewPresized( %d );" % (to_name, len(pairs)))
-
-            context.addCleanupTempName(to_name)
-
-        needs_check = not pair.getKey().isKnownToBeHashable()
-        res_name = context.getIntResName()
-
-        emit(
-            "%s = PyDict_SetItem(%s, %s, %s);"
-            % (res_name, to_name, dict_key_name, dict_value_name)
-        )
-
-        if context.needsCleanup(dict_value_name):
-            emit("Py_DECREF(%s);" % dict_value_name)
-            context.removeCleanupTempName(dict_value_name)
-
-        if context.needsCleanup(dict_key_name):
-            emit("Py_DECREF(%s);" % dict_key_name)
+        key_needs_release = context.needsCleanup(dict_key_name)
+        if key_needs_release:
             context.removeCleanupTempName(dict_key_name)
 
-        getErrorExitBoolCode(
-            condition="%s != 0" % res_name,
-            needs_check=needs_check,
-            emit=emit,
-            context=context,
-        )
+        value_needs_release = context.needsCleanup(dict_value_name)
+        if value_needs_release:
+            context.removeCleanupTempName(dict_value_name)
+
+        return key_needs_release, value_needs_release
+
+    key_needs_release, value_needs_release = generatePairCode(pairs[0])
+
+    # Create dictionary presized.
+    emit("%s = _PyDict_NewPresized( %d );" % (to_name, pairs_count))
+
+    with withCleanupFinally(
+        "dict_build", to_name, needs_exception_exit, emit, context
+    ) as guarded_emit:
+        emit = guarded_emit.emit
+
+        for count, pair in enumerate(pairs):
+            if count > 0:
+                key_needs_release, value_needs_release = generatePairCode(pair)
+
+            needs_check = not is_hashable_key[count]
+            res_name = context.getIntResName()
+
+            emit(
+                "%s = PyDict_SetItem(%s, %s, %s);"
+                % (res_name, to_name, dict_key_name, dict_value_name)
+            )
+
+            if value_needs_release:
+                emit("Py_DECREF(%s);" % dict_value_name)
+
+            if key_needs_release:
+                emit("Py_DECREF(%s);" % dict_key_name)
+
+            getErrorExitBoolCode(
+                condition="%s != 0" % res_name,
+                needs_check=needs_check,
+                emit=emit,
+                context=context,
+            )
 
 
 def generateDictOperationUpdateCode(statement, emit, context):
@@ -201,7 +247,9 @@ def generateDictOperationGetCode(to_name, expression, emit, context):
     with withObjectCodeTemporaryAssignment(
         to_name, "dict_value", expression, emit, context
     ) as value_name:
-        emit("%s = DICT_GET_ITEM(%s, %s);" % (value_name, dict_name, key_name))
+        emit(
+            "%s = DICT_GET_ITEM_WITH_ERROR(%s, %s);" % (value_name, dict_name, key_name)
+        )
 
         getErrorExitCode(
             check_name=value_name,
@@ -223,12 +271,12 @@ def generateDictOperationInCode(to_name, expression, emit, context):
 
     res_name = context.getIntResName()
 
-    emit("%s = PyDict_Contains(%s, %s);" % (res_name, key_name, dict_name))
+    emit("%s = DICT_HAS_ITEM(%s, %s);" % (res_name, key_name, dict_name))
 
     getErrorExitBoolCode(
         condition="%s == -1" % res_name,
         release_names=(dict_name, key_name),
-        needs_check=expression.mayRaiseException(BaseException),
+        needs_check=expression.known_hashable_key is not True,
         emit=emit,
         context=context,
     )
@@ -280,12 +328,12 @@ def generateDictOperationSetCode(statement, emit, context):
 
 
 def generateDictOperationSetCodeKeyValue(statement, emit, context):
-    key_arg_name = context.allocateTempName("dictset_key", unique=True)
+    key_arg_name = context.allocateTempName("dictset38_key")
     generateExpressionCode(
         to_name=key_arg_name, expression=statement.getKey(), emit=emit, context=context
     )
 
-    value_arg_name = context.allocateTempName("dictset_value", unique=True)
+    value_arg_name = context.allocateTempName("dictset38_value")
     generateExpressionCode(
         to_name=value_arg_name,
         expression=statement.getValue(),
@@ -293,7 +341,7 @@ def generateDictOperationSetCodeKeyValue(statement, emit, context):
         context=context,
     )
 
-    dict_arg_name = context.allocateTempName("dictset_dict", unique=True)
+    dict_arg_name = context.allocateTempName("dictset38_dict")
     generateExpressionCode(
         to_name=dict_arg_name,
         expression=statement.getDict(),
@@ -335,7 +383,7 @@ def generateDictOperationRemoveCode(statement, emit, context):
 
     old_source_ref = context.setCurrentSourceCodeReference(
         statement.getKey().getSourceReference()
-        if Options.isFullCompat()
+        if Options.is_fullcompat
         else statement.getSourceReference()
     )
 
