@@ -312,7 +312,7 @@ def generateTryNextExceptStopIterationCode(statement, emit, context):
 
     old_source_ref = context.setCurrentSourceCodeReference(
         assign_source.getSourceReference()
-        if Options.isFullCompat()
+        if Options.is_fullcompat
         else statement.getSourceReference()
     )
 
