@@ -32,12 +32,13 @@ from .FileOperations import withMadeWritableFileMode
 from .Utils import getArchitecture, getOS, isAlpineLinux, isWin32Windows
 from .WindowsResources import (
     RT_MANIFEST,
+    VsFixedFileInfoStructure,
     deleteWindowsResources,
     getResourcesFromDLL,
 )
 
 
-def localDLLFromFilesystem(name, paths):
+def locateDLLFromFilesystem(name, paths):
     for path in paths:
         for root, _dirs, files in os.walk(path):
             if name in files:
@@ -71,7 +72,7 @@ def locateDLL(dll_name):
             return dll_name
 
     if isAlpineLinux():
-        return localDLLFromFilesystem(
+        return locateDLLFromFilesystem(
             name=dll_name, paths=["/lib", "/usr/lib", "/usr/local/lib"]
         )
 
@@ -99,7 +100,7 @@ def locateDLL(dll_name):
 
 
 def getSxsFromDLL(filename, with_data=False):
-    """ List the SxS manifests of a Windows DLL.
+    """List the SxS manifests of a Windows DLL.
 
     Args:
         filename: Filename of DLL to investigate
@@ -110,12 +111,12 @@ def getSxsFromDLL(filename, with_data=False):
     """
 
     return getResourcesFromDLL(
-        filename=filename, resource_kind=RT_MANIFEST, with_data=with_data
+        filename=filename, resource_kinds=(RT_MANIFEST,), with_data=with_data
     )
 
 
 def removeSxsFromDLL(filename):
-    """ Remove the Windows DLL SxS manifest.
+    """Remove the Windows DLL SxS manifest.
 
     Args:
         filename: Filename to remove SxS manifests from
@@ -136,10 +137,10 @@ def removeSxsFromDLL(filename):
 
 
 def getWindowsDLLVersion(filename):
-    """ Return DLL version information from a file.
+    """Return DLL version information from a file.
 
-        If not present, it will be (0, 0, 0, 0), otherwise it will be
-        a tuple of 4 numbers.
+    If not present, it will be (0, 0, 0, 0), otherwise it will be
+    a tuple of 4 numbers.
     """
     # Get size needed for buffer (0 if no info)
     import ctypes.wintypes
@@ -191,23 +192,6 @@ def getWindowsDLLVersion(filename):
     ]
     VerQueryValueA.restype = ctypes.wintypes.BOOL
 
-    class VsFixedFileInfoStructure(ctypes.Structure):
-        _fields_ = [
-            ("dwSignature", ctypes.c_uint32),  # 0xFEEF04BD
-            ("dwStructVersion", ctypes.c_uint32),
-            ("dwFileVersionMS", ctypes.c_uint32),
-            ("dwFileVersionLS", ctypes.c_uint32),
-            ("dwProductVersionMS", ctypes.c_uint32),
-            ("dwProductVersionLS", ctypes.c_uint32),
-            ("dwFileFlagsMask", ctypes.c_uint32),
-            ("dwFileFlags", ctypes.c_uint32),
-            ("dwFileOS", ctypes.c_uint32),
-            ("dwFileType", ctypes.c_uint32),
-            ("dwFileSubtype", ctypes.c_uint32),
-            ("dwFileDateMS", ctypes.c_uint32),
-            ("dwFileDateLS", ctypes.c_uint32),
-        ]
-
     file_info = ctypes.POINTER(VsFixedFileInfoStructure)()
     uLen = ctypes.c_uint32(ctypes.sizeof(file_info))
 
@@ -226,7 +210,7 @@ def getWindowsDLLVersion(filename):
 
 # TODO: Relocate this to nuitka.freezer maybe.
 def getPEFileInformation(filename):
-    """ Return the PE file information of a Windows EXE or DLL
+    """Return the PE file information of a Windows EXE or DLL
 
     Args:
         filename - The file to be investigated.
@@ -291,7 +275,7 @@ def getPEFileInformation(filename):
 
 
 def callInstallNameTool(filename, mapping):
-    """ Update the macOS shared library information for a binary or shared library.
+    """Update the macOS shared library information for a binary or shared library.
 
     Args:
         filename - The file to be modified.
@@ -339,3 +323,26 @@ def callInstallNameToolAddRPath(filename, rpath):
 
     if result != 0:
         sys.exit("Error, call to 'install_name_tool' to add rpath failed.")
+
+
+def getPyWin32Dir():
+    """Find the pywin32 DLL directory
+
+    Args:
+        None
+
+    Returns:
+        path to the pywin32 DLL directory or None
+
+    Notes:
+        This is needed for standalone mode only.
+    """
+
+    for path_element in sys.path:
+        if not path_element:
+            continue
+
+        candidate = os.path.join(path_element, "pywin32_system32")
+
+        if os.path.isdir(candidate):
+            return candidate
